/* $Id: op_getattr.c,v 1.1 2005/12/19 18:55:12 lace Exp $
 * Client fuse interface operation "getattr" for libcaptive
 * Copyright (C) 2005 Jan Kratochvil <project-captive@jankratochvil.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; exactly version 2 of June 1991 is required
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "config.h"

#include <glib/gmessages.h>
#include <fuse.h>
#include <errno.h>
#include <captive/client-file.h>
#include <sys/stat.h>
#include <captive/macros.h>

#include "op_getattr.h"	/* self */
#include "main.h"
#include "gnomevfsresult.h"
#include "gnomevfsfileinfo.h"


int op_getattr(const char *path,struct stat *buf)
{
CaptiveFileObject *captive_file_object;
GnomeVFSResult errvfsresult;
GnomeVFSFileInfo file_info;
int errint;

	g_return_val_if_fail(path!=NULL,-EINVAL);
	g_return_val_if_fail(buf!=NULL,-EINVAL);

	if (GNOME_VFS_OK!=(errvfsresult=captive_file_new_open(&captive_file_object,capfuse_captive_vfs_object,path,0)))
		return -gnomevfsresult_to_errno(errvfsresult);

	/* Pre-clear to take care of any fields not touched by libcaptive there. */
	CAPTIVE_MEMZERO(&file_info);
	errvfsresult=captive_file_file_info_get(captive_file_object,&file_info);

	g_object_unref(captive_file_object);
	if (GNOME_VFS_OK!=errvfsresult)
		return -gnomevfsresult_to_errno(errvfsresult);
	errint=gnomevfsfileinfo_to_stat(buf,&file_info);
	gnome_vfs_file_info_clear(&file_info);

	return errint;
}
